# Copyright (c) 2025, UChicago Argonne, LLC
# BSD OPEN SOURCE LICENSE. Full license can be found in LICENSE.md
from polaris.network.create.triggers import recreate_network_triggers


def migrate(conn):

    to_del = [
        "prevent_delete_version",
        "prevent_change_version",
        "connection_populates_node_todir_geo_new_record",
        "connection_enforce_node_on_table_change",
        "connection_enforce_todir_on_table_change",
        "counties_populates_geo_enabled_fields_on_new_record",
        "counties_populates_geo_enabled_fields_on_new_record",
        "counties_enforces_x_field_Counties",
        "counties_enforces_y_field_Counties",
        "counties_updates_fields_on_geo_change",
        "evs_populates_fields_on_new_record",
        "evs_enforces_longitude_field",
        "evs_enforces_latitude_field",
        "evs_updates_lat_lon_fields_on_geo_change",
        "evs_delete_all_data_for_deleted_stations",
        "evs_updates_zone_field",
        "evs_updates_location_field",
        "land_use_is_work_update",
        "land_use_is_work_insert",
        "land_use_is_home_update",
        "land_use_is_home_insert",
        "land_use_is_discretionary_update",
        "land_use_is_discretionary_insert",
        "land_use_is_school_update",
        "land_use_is_school_insert",
        "link_overrides_new_override",
        "link_overrides_change_field",
        "location_populates_fields_on_new_record",
        "location_enforces_x_field_Location",
        "location_enforces_setback_field_Location",
        "location_enforces_link_field_Location",
        "location_enforces_y_field_Location",
        "location_updates_xyz_fields_on_geo_change",
        "location_updates_area_type_on_zone_change",
        "location_enforces_area_type_on_area_type_change",
        "location_on_link_change",
        "location_on_walk_link_change",
        "location_on_bike_link_change",
        "location_on_walk_offset_change",
        "location_on_bike_offset_change",
        "location_deleted_location",
        "location_links_creates_distance_field_on_insert",
        "location_links_updates_distance_field_on_change",
        "location_links_deletes",
        "location_parking_creates_distance_field_on_insert",
        "location_parking_updates_distance_field_on_change",
        "micromobility_docks_populates_xyz_fields_on_new_record",
        "micromobility_docks_enforces_x_field",
        "micromobility_docks_enforces_y_field",
        "micromobility_docks_enforces_zone_field",
        "micromobility_docks_enforces_link_field",
        "micromobility_docks_on_geo_change",
        "micromobility_unique_tag",
        "transit_agencies_unique_tag",
        "network_updated_link",
        "network_eliminates_ab_direction",
        "network_eliminates_ba_direction",
        "network_changes_ab_lanes",
        "network_changes_ba_lanes",
        "network_fspd_ab_only_positive_for_available_direction",
        "network_fspd_ba_only_positive_for_available_direction",
        "network_cap_ab_only_positive_for_available_direction",
        "network_cap_ba_only_positive_for_available_direction",
        "network_preserves_cap_ab_direction",
        "network_preserves_cap_ba_direction",
        "network_preserves_fspd_ba_direction",
        "link_queue_edit_delete_link",
        "link_queue_edit_add_link",
        "link_queue_edit_change_geo_link",
        "link_queue_edit_change_type_link",
        "node_queue_edit_delete",
        "network_changes_on_zone_node",
        "network_changes_on_link_area_type",
        "network_update_link_toll_counterpart",
        "network_prevent_delete_toll_counterpart",
        "network_delete_referring_records",
        "cleans_up_after_removing_node",
        "network_new_link_node_a",
        "network_new_link_node_b",
        "network_update_link_node_a",
        "network_update_link_node_b",
        "network_new_link",
        "network_enforces_link_bearing_a_update",
        "network_enforces_link_bearing_b_update",
        "network_enforces_link_length_update",
        "network_updated_link_geo",
        "network_deleted_link",
        "network_update_node_geo",
        "network_cannibalise_node",
        "network_no_duplicate_node",
        "network_dont_delete_node",
        "network_populate_node_fields_on_insert",
        "network_enforces_x_field_node",
        "network_enforces_y_field_node",
        "network_updated_node",
        "network_updated_node_fix_index",
        "parking_enforces_setback_field_setback",
        "parking_enforces_link_field_link",
        "parking_updates_fields_on_geo_change",
        "parking_on_walk_link_change",
        "parking_on_walk_offset_change",
        "parking_on_bike_link_change",
        "parking_on_bike_offset_change",
        "parking_deleted_parking",
        "parking_updates_zone_field",
        "Road_Connector_populates_length_on_new_record",
        "Road_Connector_populates_length_on_new_geometry",
        "Road_Connectorenforces_length_field",
        "Road_Connectorenforces_bearing",
        "signal_populates_times_new_record",
        "signal_populates_times_change_in_table",
        "signal_nested_populates_times_new_record",
        "signal_nested_populates_change_in_table",
        "signal_nested_populates_delete_in_table",
        "phasing_nested_populates_movements_new_record",
        "phasing_nested_populates_movements_change_in_table",
        "phasing_nested_populates_movements_delete_in_table",
        "phasing_progagates_delete",
        "phasing_populates_movements_new_record",
        "phasing_populates_movements_change_in_table",
        "signal_delete_in_table",
        "timing_nested_updates_fields_on_insert_in_table",
        "timing_nested_updates_fields_on_update_in_table",
        "timing_nested_updates_fields_on_delete_in_table",
        "timing_progagates_delete",
        "timing_updates_fields_on_update_in_table",
        "timing_updates_fields_on_insert_in_table",
        "sign_populates_new_record",
        "sign_stop_on_freeway",
        "transit_bike_queue_edit_delete_link",
        "transit_bike_queue_edit_add_link",
        "transit_bike_queue_edit_change_geo_link",
        "transit_bike_change_bike_link_id",
        "transit_bike_enforces_link_bearing_a_update",
        "transit_bike_enforces_link_bearing_b_update",
        "transit_delete_patterns_when_deleting_routes",
        "transit_delete_trips_when_deleting_patterns",
        "transit_delete_when_deleting_trips",
        "transit_delete_connector_when_delete_stop",
        "transit_stops_populates_xyz_fields_on_new_record",
        "transit_stops_enforces_x_field",
        "transit_stops_enforces_y_field",
        "transit_stops_enforces_zone_field",
        "transit_stops_on_geo_change",
        "transit_stops_dont_delete_stop",
        "transit_stops_only_delete_needed_stop",
        "transit_links_populates_length_on_new_record",
        "transit_links_populates_length_on_new_geometry",
        "transit_links_enforces_length_field",
        "transit_pattern_mapping_rounds_offset_on_change",
        "transit_pattern_mapping_rounds_offset_on_insert",
        "transit_walk_queue_edit_delete_link",
        "transit_walk_queue_edit_add_link",
        "transit_walk_queue_edit_change_geo_link",
        "transit_walk_change_walk_link_id",
        "transit_walk_enforces_link_bearing_a_update",
        "transit_walk_enforces_link_bearing_b_update",
        "turn_overrides_rejects_if_not_connecting_insert",
        "turn_overrides_populates_node_todir_new_record",
        "turn_overrides_enforce_node_todir_on_table_change",
        "turn_overrides_rejects_if_not_connecting_update",
        "zone_queue_change_deletion",
        "zone_populates_geo_enabled_fields_on_new_record",
        "zone_enforces_x_field_Zone",
        "zone_enforces_area_field_Zone",
        "zone_enforces_y_field_Zone",
        "zone_updates_xyz_fields_on_geo_change",
        "zone_renumbering",
        "zone_change_area_type",
    ]

    for trigger in to_del:
        conn.execute(f"DROP TRIGGER IF EXISTS {trigger}")
    recreate_network_triggers(conn)
